package com.bomostory.sceneeditmodule.cover

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Bundle
import android.support.design.widget.TabLayout
import android.support.v4.app.DialogFragment
import android.support.v4.app.FragmentManager
import android.support.v4.view.PagerAdapter
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.bomostory.sceneeditmodule.SceneDrawer
import com.bomostory.sceneeditmodule.basicdata.Project
import com.bomostory.sceneeditmodule.utils.FileUtils
import com.example.tfat.myapplication.R
import kotlinx.android.synthetic.main.dialog_cover_editor.*
import kotlinx.android.synthetic.main.dialog_cover_editor.view.*
import java.io.File
import java.io.FileOutputStream

class CoverEditorDialog : DialogFragment() {
    companion object {
        const val FRONT_PAGE_POSITION = 0
        const val PAGE_COUNT = 2
    }

    init {
        isCancelable = false
    }

    private val dialogTag = this::class.java.simpleName

    lateinit var onViewCreated: Runnable
    var onSave: View.OnClickListener? = null
        set(value) {
            field = value
            frontCoverEditorView_coverEditorDialog.onSave = value
            backCoverEditorView_coverEditorDialog.onSave = value
        }
    var onCancel: View.OnClickListener? = null
        set(value) {
            field = value
            frontCoverEditorView_coverEditorDialog.onCancel = value
            backCoverEditorView_coverEditorDialog.onCancel = value
        }
    var storyName: String
        get () {
            return frontCoverEditorView_coverEditorDialog.storyName
        }
        set(value) {
            frontCoverEditorView_coverEditorDialog.storyName = value
        }
    var author: String
        get() {
            return frontCoverEditorView_coverEditorDialog.author
        }
        set(value) {
            frontCoverEditorView_coverEditorDialog.author = value
        }
    var frontCoverColor: CoverColor
        get() {
            return frontCoverEditorView_coverEditorDialog.selectedColor
        }
        set(value) {
            frontCoverEditorView_coverEditorDialog.selectedColor = value
        }
    var backCoverColor: CoverColor
        get() {
            return backCoverEditorView_coverEditorDialog.selectedColor
        }
        set(value) {
            backCoverEditorView_coverEditorDialog.selectedColor = value
        }
    var category: Category
        get() {
            return frontCoverEditorView_coverEditorDialog.category
        }
        set(value) {
            frontCoverEditorView_coverEditorDialog.category = value
        }
    var project: Project? = null
    var coverFile: File? = null
        set(value) {
            field = value
            if (value != null) {
                val bitmap = BitmapFactory.decodeFile(value.absolutePath)
                frontCoverEditorView_coverEditorDialog.sceneCover = bitmap
            }
        }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.dialog_cover_editor, container, false).apply {
            val viewPager = viewPager_coverEditorDialog_content
            val tabLayout = tabLayout_coverEditorDialog_header
            val frontCoverEditorView = frontCoverEditorView_coverEditorDialog
            viewPager.adapter = object : PagerAdapter() {
                override fun instantiateItem(container: ViewGroup, position: Int): Any {
                    return container.getChildAt(position)
                }

                override fun isViewFromObject(p0: View, p1: Any): Boolean {
                    return p0 == p1
                }

                override fun getCount(): Int {
                    return PAGE_COUNT
                }

                override fun destroyItem(container: ViewGroup, position: Int, `object`: Any) {
                    // no super
                }
            }
            viewPager.addOnPageChangeListener(TabLayout.TabLayoutOnPageChangeListener(tabLayout))
            tabLayout.apply {
                addTab(newTab().setText(R.string.cover_editor_dialog_tab_front))
                addTab(newTab().setText(R.string.cover_editor_dialog_tab_back))
                addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
                    override fun onTabReselected(tab: TabLayout.Tab?) {}
                    override fun onTabUnselected(tab: TabLayout.Tab?) {}
                    override fun onTabSelected(tab: TabLayout.Tab?) {
                        tab?.let { viewPager.currentItem = it.position }
                    }
                })
            }
            frontCoverEditorView.apply {
                this.onColorSelectedListenr = Runnable {
                    coverFile = null
                }
                this.onSave = this@CoverEditorDialog.onSave
                this.onCancel = this@CoverEditorDialog.onCancel
                this.onChooseScene = View.OnClickListener {
                    val project = this@CoverEditorDialog.project ?: return@OnClickListener
                    this@CoverEditorDialog.frontCoverSceneChooser_frontCoverEditor.setProjectAndSelected(project, 0)
                    this@CoverEditorDialog.frontCoverSceneChooser_frontCoverEditor.title = storyName
                    this@CoverEditorDialog.frontCoverSceneChooser_frontCoverEditor.author = author
                    this@CoverEditorDialog.frontCoverSceneChooser_frontCoverEditor.visibility = View.VISIBLE
                }
            }
            frontCoverSceneChooser_frontCoverEditor.apply {
                this.onApply = View.OnClickListener {
                    this@CoverEditorDialog.frontCoverSceneChooser_frontCoverEditor.visibility = View.GONE
                    val selectedSceneIndex = this@CoverEditorDialog.frontCoverSceneChooser_frontCoverEditor.selected
                    val scenes = this@CoverEditorDialog.project?.story?.scenes ?: return@OnClickListener
                    val scene = scenes[selectedSceneIndex]
                    val width = context.resources.getDimension(R.dimen.share_dialog_screenshot_width).toInt()
                    val height = context.resources.getDimension(R.dimen.share_dialog_screenshot_height).toInt()
                    val bitmap = SceneDrawer.drawScene(context, scene, 0, width, height) ?: return@OnClickListener
                    val file = FileUtils.getCoverFile(project!!)
                    val fos = FileOutputStream(file)
                    bitmap.compress(Bitmap.CompressFormat.PNG, 0, fos)
                    fos.flush()
                    fos.close()
                    frontCoverEditorView.sceneCover = bitmap
                    coverFile = file
                }
                this.onDiscard = View.OnClickListener {
                    this@CoverEditorDialog.frontCoverSceneChooser_frontCoverEditor.visibility = View.GONE
                }
            }
            backCoverEditorView_coverEditorDialog.apply {
                this.onSave = this@CoverEditorDialog.onSave
                this.onCancel = this@CoverEditorDialog.onCancel
            }
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        onViewCreated.run()
    }

    fun show(manager: FragmentManager?) {
        manager?.apply {
            val ft = beginTransaction()
            findFragmentByTag(dialogTag)?.let { ft.remove(it) }
            ft.addToBackStack(null)
            ft.commit()
        }
        super.show(manager, dialogTag)
    }

    fun showDuplicatedProjectNameMsg() {
        viewPager_coverEditorDialog_content.currentItem = FRONT_PAGE_POSITION
        frontCoverEditorView_coverEditorDialog.showDuplicatedProjectNameMsg()
    }

    fun showEmptyProjectNameMsg() {
        viewPager_coverEditorDialog_content.currentItem = FRONT_PAGE_POSITION
        frontCoverEditorView_coverEditorDialog.showEmptyProjectNameMsg()
    }
}