package com.bomostory.sceneeditmodule.screen.view

import android.content.Context
import android.graphics.*
import android.util.AttributeSet
import android.view.View
import android.graphics.drawable.VectorDrawable
import android.graphics.Bitmap
import android.util.DisplayMetrics
import android.view.MotionEvent
import android.view.WindowManager
import android.widget.RelativeLayout
import com.bomostory.sceneeditmodule.DialogueDrawer
import com.bomostory.sceneeditmodule.basicdata.Actor
import kotlinx.android.synthetic.main.actor_view.view.*

class DialogueView : EditActorView{

    constructor(context: Context) : super(context)

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)

    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)

    val radius = 25f
    val MIN_DIALOGUE_WIDTH = (400 - radius * 2) / 2f + radius * 2
    val MIN_DIALOGUE_HEIGHT = (320 - radius * 2) / 2f + radius * 2
    val DIALOGUE_RATIO = MIN_DIALOGUE_HEIGHT / MIN_DIALOGUE_WIDTH

    var dialogue = Actor()
        set(value) {
            value?.let {
                field = value
                val layoutParams = RelativeLayout.LayoutParams(it.sideLength, it.sideHeight)
                layoutParams.leftMargin = it.positionX
                layoutParams.topMargin = it.positionY
                actor_layout.layoutParams = layoutParams
                val actorLayoutParams = RelativeLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
                actor_view_layout.layoutParams = actorLayoutParams
                when(it.isMovable) {
                    true -> actor_view_layout.setOnTouchListener(actorOnTouchListener())
                }
                when(it.isSelect){
                    true -> change_size_view.visibility = View.VISIBLE
                    false -> change_size_view.visibility = View.GONE
                }
            }
        }


    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        canvas?.drawBitmap(DialogueDrawer.drawDialogue(context, dialogue), dialogue.positionX.toFloat(), dialogue.positionY.toFloat(), null)
        if(dialogue.isSelect) {
            val mPaint = Paint()
            mPaint.style = Paint.Style.STROKE
            mPaint.strokeWidth = 5f
            mPaint.isAntiAlias = true
            mPaint.color = Color.parseColor("#4e342e")
            mPaint.pathEffect = DashPathEffect(floatArrayOf(8f, 8f), 0f)
            canvas?.drawLine(dialogue.positionX.toFloat() + 25 , dialogue.positionY.toFloat() + 25, dialogue.positionX.toFloat() + 25, dialogue.positionY.toFloat() + 25 + dialogue.sideHeight , mPaint)
            canvas?.drawLine(dialogue.positionX.toFloat() + 25, dialogue.positionY.toFloat() + 25, dialogue.positionX.toFloat() + dialogue.sideLength.toFloat() - 25, dialogue.positionY.toFloat() + 25, mPaint)
            canvas?.drawLine(dialogue.positionX.toFloat() + 25, dialogue.positionY.toFloat() + dialogue.sideHeight.toFloat() - 25, dialogue.positionX.toFloat()  + dialogue.sideLength.toFloat() + 25, dialogue.positionY.toFloat() + dialogue.sideHeight.toFloat() - 25, mPaint)
            canvas?.drawLine(dialogue.positionX.toFloat() + dialogue.sideLength.toFloat() - 25, dialogue.positionY.toFloat() + 25, dialogue.positionX.toFloat() + dialogue.sideLength.toFloat() - 25, dialogue.positionY.toFloat() + dialogue.sideHeight.toFloat() - 25, mPaint)
        }
    }

    private fun actorOnTouchListener() : View.OnTouchListener {
        var clickPeriod = 0L
        var positionStart = 0
        var positionEnd = 0
        var mx: Int
        var my: Int
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    clickPeriod = System.currentTimeMillis()
                    positionStart = (motionEvent.x).toInt()
                    positionEnd = (motionEvent.y).toInt()
                }
                MotionEvent.ACTION_MOVE -> {
                    if (!(Math.abs(motionEvent.x - positionStart) < 10 && Math.abs(motionEvent.y - positionEnd) < 10) && dialogue.isSelect) {
                        mx = motionEvent.rawX.toInt() - positionStart
                        my = motionEvent.rawY.toInt() - positionEnd
                        dialogue.positionX = Math.max(0, Math.min(SCREEN_WIDTH - dialogue.sideLength, mx))
                        dialogue.positionY = Math.max(0, Math.min(SCREEN_HEIGHT - dialogue.sideHeight, my - 200))
                        var laParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                        laParams.setMargins(dialogue.positionX, dialogue.positionY, 0, 0)
                        actor_layout.layoutParams = laParams
                        val actorLayoutParams = RelativeLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
                        view.layoutParams = actorLayoutParams
                    }
                }
                MotionEvent.ACTION_UP -> {
                    if (Math.abs(motionEvent.x - positionStart) < 10 && Math.abs(motionEvent.y - positionEnd) < 10 && (System.currentTimeMillis() - clickPeriod) > 1000) {
                        actorCallback?.onActorLongClick(dialogue)
                    } else if (Math.abs(motionEvent.x - positionStart) < 10 && Math.abs(motionEvent.y - positionEnd) < 10) {
                        actorCallback?.onActorSelected(dialogue)
                    } else {
                        if (dialogue.isSelect){
                            actorCallback?.onActorChange(dialogue)
                        }
                    }
                }
            }
            return@OnTouchListener true
        }
    }

    override fun rightBottomSizeChangeListener() : View.OnTouchListener {
        var positionX = 0f
        var positionY = 0f
        var sizeX = 0f
        var sizeY = 0f
        var x = 0
        var y = 0
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    val layoutParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                    sizeX = layoutParams.width.toFloat()
                    sizeY = layoutParams.height.toFloat()
                    x = layoutParams.leftMargin
                    y = layoutParams.topMargin
                    positionX = motionEvent.rawX
                    positionY = motionEvent.rawY
                }
                MotionEvent.ACTION_MOVE -> {
                    var mx = Math.min(SCREEN_WIDTH - CONTROLLER_RADIUS.toFloat(), motionEvent.rawX) - positionX
                    var my = Math.min(SCREEN_HEIGHT + SCREEN_TOP_MARGIN - CONTROLLER_RADIUS.toFloat(), motionEvent.rawY) - positionY
                    var dP = when (my < mx){
                        true -> my
                        false -> mx
                    }
                    var dialogueData = Actor().apply {
                        when (my < mx){
                            true -> {
                                sideLength = Math.max(MIN_DIALOGUE_WIDTH, sizeX + dP / DIALOGUE_RATIO).toInt()
                                sideHeight = Math.max(MIN_DIALOGUE_HEIGHT, sizeY + dP).toInt()
                                this.positionX = x
                                this.positionY = y
                            }
                            false -> {
                                sideLength = Math.max(MIN_DIALOGUE_WIDTH, sizeX + dP).toInt()
                                sideHeight = Math.max(MIN_DIALOGUE_HEIGHT, sizeY + dP * DIALOGUE_RATIO).toInt()
                                this.positionX = x
                                this.positionY = y
                            }
                        }
                        resourcePath = dialogue.resourcePath
                        text = dialogue.text
                        textColor = dialogue.textColor
                        textAlign = dialogue.textAlign
                        positionZ = dialogue.positionZ
                        isSelect = dialogue.isSelect
                        parentLayerIndex = dialogue.parentLayerIndex
                        isMovable = dialogue.isMovable
                        isDialogue = dialogue.isDialogue
                        isMirror = dialogue.isMirror
                        opacity = actor.opacity
                    }
                    dialogue = dialogueData
                }
                MotionEvent.ACTION_UP -> {
                    dialogue.isSelect = true
                    actorCallback?.onActorChange(dialogue)
                }
            }
            return@OnTouchListener true
        }
    }

    override fun leftTopSizeChangeListener() : View.OnTouchListener {
        var positionX = 0f
        var positionY = 0f
        var sizeX = 0f
        var sizeY = 0f
        var x = 0
        var y = 0
        var rightX = 0f
        var bottomY = 0f
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    val layoutParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                    sizeX = layoutParams.width.toFloat()
                    sizeY = layoutParams.height.toFloat()
                    x = layoutParams.leftMargin
                    y = layoutParams.topMargin
                    positionX = motionEvent.rawX
                    positionY = motionEvent.rawY
                    rightX = x + sizeX
                    bottomY = y + sizeY
                }
                MotionEvent.ACTION_MOVE -> {
                    val mx = Math.max(CONTROLLER_RADIUS.toFloat(), motionEvent.rawX) - positionX
                    var my = Math.max(SCREEN_TOP_MARGIN + CONTROLLER_RADIUS.toFloat(), motionEvent.rawY) - positionY
                    var dP = when (my > mx){
                        true -> my
                        false -> mx
                    }
                    var dialogueData = Actor().apply {
                        when (my > mx){
                            true -> {
                                sideLength = Math.max(MIN_DIALOGUE_WIDTH, sizeX - dP / DIALOGUE_RATIO).toInt()
                                sideHeight = Math.max(MIN_DIALOGUE_HEIGHT, sizeY - dP).toInt()
                                this.positionX = Math.min(rightX - MIN_DIALOGUE_WIDTH, x + dP / DIALOGUE_RATIO).toInt()
                                this.positionY = Math.min(bottomY - MIN_DIALOGUE_HEIGHT, y + dP).toInt()
                            }
                            false -> {
                                sideLength = Math.max(MIN_DIALOGUE_WIDTH, sizeX - dP).toInt()
                                sideHeight = Math.max(MIN_DIALOGUE_HEIGHT, sizeY - dP * DIALOGUE_RATIO).toInt()
                                this.positionX = Math.min(rightX - MIN_DIALOGUE_WIDTH, x + dP).toInt()
                                this.positionY = Math.min(bottomY - MIN_DIALOGUE_HEIGHT, y + dP * DIALOGUE_RATIO).toInt()
                            }
                        }
                        resourcePath = dialogue.resourcePath
                        text = dialogue.text
                        textColor = dialogue.textColor
                        textAlign = dialogue.textAlign
                        positionZ = dialogue.positionZ
                        isSelect = dialogue.isSelect
                        parentLayerIndex = dialogue.parentLayerIndex
                        isMovable = dialogue.isMovable
                        isDialogue = dialogue.isDialogue
                        isMirror = dialogue.isMirror
                        opacity = actor.opacity
                    }
                    dialogue = dialogueData
                }
                MotionEvent.ACTION_UP -> {
                    dialogue.isSelect = true
                    actorCallback?.onActorChange(dialogue)
                }
            }
            return@OnTouchListener true
        }
    }

    override fun rightTopSizeChangeListener() : View.OnTouchListener {
        var positionX = 0f
        var positionY = 0f
        var sizeX = 0f
        var sizeY = 0f
        var x = 0
        var y = 0
        var bottomY = 0f
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    val layoutParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                    sizeX = layoutParams.width.toFloat()
                    sizeY = layoutParams.height.toFloat()
                    x = layoutParams.leftMargin
                    y = layoutParams.topMargin
                    positionX = motionEvent.rawX
                    positionY = motionEvent.rawY
                    bottomY = y + sizeY
                }
                MotionEvent.ACTION_MOVE -> {
                    var mx = Math.min(SCREEN_WIDTH - CONTROLLER_RADIUS.toFloat(), motionEvent.rawX) - positionX
                    var my = Math.max(SCREEN_TOP_MARGIN + CONTROLLER_RADIUS.toFloat(), motionEvent.rawY) - positionY
                    var dP = when (-mx < my){
                        true ->  my
                        false -> -mx
                    }
                    var dialogueData = Actor().apply {
                        when (-mx < my){
                            true -> {
                                sideLength = Math.max(MIN_DIALOGUE_WIDTH, sizeX - dP / DIALOGUE_RATIO).toInt()
                                sideHeight = Math.max(MIN_DIALOGUE_HEIGHT, sizeY - dP).toInt()
                                this.positionX = x
                                this.positionY = Math.min(bottomY - MIN_DIALOGUE_HEIGHT, y + dP).toInt()
                            }
                            false -> {
                                sideLength = Math.max(MIN_DIALOGUE_WIDTH, sizeX - dP).toInt()
                                sideHeight = Math.max(MIN_DIALOGUE_HEIGHT, sizeY - dP * DIALOGUE_RATIO).toInt()
                                this.positionX = x
                                this.positionY = Math.min(bottomY - MIN_DIALOGUE_HEIGHT, y + dP * DIALOGUE_RATIO).toInt()
                            }
                        }
                        resourcePath = dialogue.resourcePath
                        text = dialogue.text
                        textColor = dialogue.textColor
                        textAlign = dialogue.textAlign
                        positionZ = dialogue.positionZ
                        isSelect = dialogue.isSelect
                        parentLayerIndex = dialogue.parentLayerIndex
                        isMovable = dialogue.isMovable
                        isDialogue = dialogue.isDialogue
                        isMirror = dialogue.isMirror
                        opacity = actor.opacity
                    }
                    dialogue = dialogueData
                }
                MotionEvent.ACTION_UP -> {
                    dialogue.isSelect = true
                    actorCallback?.onActorChange(dialogue)
                }
            }
            return@OnTouchListener true
        }
    }

    override fun leftBottomSizeChangeListener() : View.OnTouchListener {
        var positionX = 0f
        var positionY = 0f
        var sizeX = 0f
        var sizeY = 0f
        var x = 0
        var y = 0
        var rightX = 0f
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    val layoutParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                    sizeX = layoutParams.width.toFloat()
                    sizeY = layoutParams.height.toFloat()
                    x = layoutParams.leftMargin
                    y = layoutParams.topMargin
                    positionX = motionEvent.rawX
                    positionY = motionEvent.rawY
                    rightX = x + sizeX
                }
                MotionEvent.ACTION_MOVE -> {
                    val mx = Math.max(CONTROLLER_RADIUS.toFloat(), motionEvent.rawX) - positionX
                    var my = Math.min(SCREEN_HEIGHT + SCREEN_TOP_MARGIN - CONTROLLER_RADIUS.toFloat(), motionEvent.rawY) - positionY
                    var dP = when (-mx > my){
                        true -> my
                        false -> -mx
                    }
                    var dialogueData = Actor().apply {
                        when (-mx > my){
                            true -> {
                                sideLength = Math.max(MIN_DIALOGUE_WIDTH, sizeX + dP / DIALOGUE_RATIO).toInt()
                                sideHeight = Math.max(MIN_DIALOGUE_HEIGHT, sizeY + dP).toInt()
                                this.positionX = Math.min(rightX - MIN_DIALOGUE_WIDTH, x - dP / DIALOGUE_RATIO).toInt()
                                this.positionY = y
                            }
                            false -> {
                                sideLength = Math.max(MIN_DIALOGUE_WIDTH, sizeX + dP).toInt()
                                sideHeight = Math.max(MIN_DIALOGUE_HEIGHT, sizeY + dP * DIALOGUE_RATIO).toInt()
                                this.positionX = Math.min(rightX - MIN_DIALOGUE_WIDTH, x - dP).toInt()
                                this.positionY = y
                            }
                        }
                        resourcePath = dialogue.resourcePath
                        text = dialogue.text
                        textColor = dialogue.textColor
                        textAlign = dialogue.textAlign
                        positionZ = dialogue.positionZ
                        isSelect = dialogue.isSelect
                        parentLayerIndex = dialogue.parentLayerIndex
                        isMovable = dialogue.isMovable
                        isDialogue = dialogue.isDialogue
                        isMirror = dialogue.isMirror
                        opacity = actor.opacity
                    }
                    dialogue = dialogueData
                }
                MotionEvent.ACTION_UP -> {
                    dialogue.isSelect = true
                    actorCallback?.onActorChange(dialogue)
                }
            }
            return@OnTouchListener true
        }
    }
}